
%
% fum2.m
%
% Returns any of 80 possible, alternative solutions to the
% steady state of the fumarase model, assuming a py-substitution
% strategy in which sets K and X are partitioned as follows:
%
%   K_p  = {k1,...,k5,k12}
%   K_qy = {k6,...,k11}
%   X_p  = {x1,...,x5}
%   X_qy = {x6,...,x9}
%
% The 80 alternative solutions are the set of all possible
% linear combinations of the null space basis vectors that
% yield a valid map psi_y : Y_q --> Q, Y_q^c --> R[P,Q],
% where
%
%         | y1 or y10
%   Y_q = < y6 or y9
%         | any three of {y2,y3,y4,y5,y7,y8}
%
% To return an alternative solution, set the positional 
% parameter alt to true. By default, alt=false, and the
% default linear combination of basis vectors is used,
% q = [ q1, q2, q3, q4, q5 ]^T. If alt=true, then a
% particular combination of basis vectors is derived
% such that Y_q = y(yi), where by default yi=[5,7,8,9,10].
% A value for yi that does not result in a set Y_q satisfying
% the membership requirements stated above will result in
% an error.
%
% Arguments:
% alt - if true, a paritcular combination of null space basis
%       vectors is identified such that set Y_q = { y(yi) }.
%       Default value is false.
% yi  - Indices of elements in y to be partitioned into the
%       set Y_q. The length of yi must be 5 and the indices
%       must identify a subset of y consistent with the
%       requirements stated in the comments, above. Default
%       value is yi=[5,7,8,9,10].
%
% Optional Arguments:
% doUnsub - if true, the py-substitution is reversed prior to
%       returning vbar. Default value is true.
% verbose - if true, each calculation in the substitution and
%       solution of the steady state equation are printed to the 
%       console. Default value is false.
% fid - a file id to which to print text when verbose=true. Returned
%       by a previous call to fopen(). Default value is 1, the console. 
%       This argument is ignored if verbose=false.
% wid - width of the device, in characters, to which to print.
%       Default value is 80.
%
% Value:
% vbar - the steady state vector of reaction velocities 
%        satisfying S * vbar = 0. Elements in vbar are 
%        either in terms of p and q (if doUnsub is false), 
%        or k and x (if doUnsub is true).
% psi  - the composite mapping function from the set 
%        union(K,X) to union(P,Q,R[P,Q]) if doUnsub is 
%        false, or to union(K_pq,X_pq) if doUnsub is true, 
%        satisfying psi(v) = vbar.

function [vbar,psi] = fum2( varargin )

  ip = inputParser;
  addOptional(ip,'alt',false);
  addOptional(ip,'yi',[5,7,8,9,10]);
  addParamValue(ip,'doUnsub',true,@islogical);
  addParamValue(ip,'verbose',false,@islogical);
  addParamValue(ip,'fid',1,@isnumeric);
  addParamValue(ip,'wid',80,@isnumeric);

  parse(ip,varargin{:});
  alt = ip.Results.alt;
  yi = ip.Results.yi;
  doUnsub = ip.Results.doUnsub;
  verbose = ip.Results.verbose;
  fid = ip.Results.fid;
  wid = ip.Results.wid;


  % Load the model description
  [S,v,k,x,xdot] = loadFum();

  if( verbose )
    printvar(sprintf(strcat('\nThere are %d reactions',...
      ' and %d species in the fumarase model.\n'),...
      length(k), length(x)), 'fid', fid, 'wid', wid );
    printvar(sprintf(strcat('The stoichiometric matrix is S =\n')),...
      'fid', fid, 'wid', wid);
    printvar( S, 'vsym', 'S', 'fid', fid, 'wid', wid );
    printvar(sprintf(strcat('\nThe vector of reaction',...
      ' velocities is v, where\n')),'wid',wid,'fid',fid);
    printvar( v, 'vsym', 'v', 'fid', fid, 'wid', wid );
    printvar(sprintf(strcat('\nThe vector of mass balance',...
      ' equations is xdot = S*v, where\n')),'wid',wid,'fid',fid);
    printvar( xdot, 'vsym', 'xdot', 'fid', fid, 'wid', wid );
  end
  
  % Declare vectors of in/dependent variables
  p = sym( 'p', [11,1] ); y = sym( 'y', [10,1] );
  p = sym( p, 'positive' ); p = sym( p, 'real' );
  y = sym( y, 'positive' ); y = sym( y, 'real' );
  
  % Substitution strategy
  kx = [ k' x' ];
  py = [ p(1:5)' y(1:6)' p(6:11)' y(7:10)' ];
  
  % Execute pysub
  [vbar,ybar,psi_py,q] = pysub( S, v, kx, py, y,...
    'verbose', false );
  
  % Alternative solution (always run this part,
  % regardless of the value of alt).
  clear eqs

  % Find multiples a of efficients q such that
  % ybar(yi(1..5)) = q1..q5
  a = sym( 'a', [5,1] ); a = sym( a, 'real' );
  ybar = subs( ybar, q, q.*a );

  for i = 1:length(yi)
    eqs{i} = (strcat(char(ybar(yi(i))),'=',char(q(i))));
  end
  avars = num2cell(a);
  asoln = solve( eqs{:}, avars{:} );
  qp = simplify( subs( q.*a, a, struct2cell(asoln) ) );
  keyboard
  [vbar,ybar,psi_py,qp] = pysub( S, v, kx, py, y, ...
    'q', qp, 'verbose', verbose, 'wid', wid, 'fid', fid );


  % Reverse the substitution
  if( doUnsub )
    [vbar,psi_ss] = unsub( v,p,q,y,kx,py,ybar,psi_py,...
      'verbose', verbose, 'fid', fid, 'wid', wid );
    psi = psi_ss;
  else
    psi = psi_py;
  end
  
  % Verify steady state
  if( verbose )
    printvar(sprintf(strcat('\nVerify steady state.',...
      ' xdot = S * vbar, where\n')), 'fid', fid, 'wid', wid); 
    printvar( simplify(S*vbar), 'vtype', 'vec', 'vsym', 'xdot',...
      'fid', fid, 'wid', wid );
  end
