
%
% loadFum.m
%
% Defines a function loadFum() that returns a description 
% of the fumarase model, by which fumarate is converted to
% malate by a 4-step enzymatic process. This model is the 
% second of three described in the Results section and is 
% used to compare py-substitution with the King-Altman 
% method.
%
% The structure of the fumarase model is as follows:
%

%  Index    Symbol    Species 
% =======  ========  =====================================
%  x1       E1        fumarase
%  x2       E2        fumarase:fumarate:proton:hydroxyl
%  x3       E3        fumarase:fumarate
%  x4       E4        fumarase:fumarate:proton
%  x5       E5        fumarase:proton
%       
%  x6       A         fumarate
%  x7       B         proton
%  x8       C         hydroxyl
%  x9       P         malate
%
%
%  Index     Reaction
% =======   ==============================================
%  v1        E1 + A  --(v1)->  E3
%  v2        E3 + B  --(v2)->  E4
%  v3        E1 + B  --(v3)->  E5
%  v4        E5 + A  --(v4)->  E4
%  v5        E4 + C  --(v5)->  E2
%  v6        E2  --(v6)->  E1 + P
%  v7        E3  --(v7)->  E1 + A
%  v8        E4  --(v8)->  E3 + B
%  v9        E5  --(v9)->  E1 + B
%  v10       E4  --(v10)->  E5 + A
%  v11       E2  --(v11)->  E4 + C
%  v12       E1 + P  --(v12)->  E2

function [S,v,k,x,dxdt] = loadFum()

  % Species and reaction rate constants
  x = sym( 'x', [9,1] ); k = sym( 'k', [12,1] );
  k = sym( k, 'positive' ); k = sym( k, 'real' );
  x = sym( x, 'positive' ); x = sym( x, 'real' );

  % Reaction velocities
  v = [ ...
    k(1)  * x(1) * x(6);
    k(2)  * x(3) * x(7);
    k(3)  * x(1) * x(7);
    k(4)  * x(5) * x(6);
    k(5)  * x(4) * x(8);
    k(6)  * x(2);
    k(7)  * x(3);
    k(8)  * x(4);
    k(9)  * x(5);
    k(10) * x(4);
    k(11) * x(2);
    k(12) * x(1) * x(9);
  ];
  
  % Stoichiometric matrix
  S = [ ...
  
   -1  0 -1  0  0  1  1  0  1  0  0 -1;
    0  0  0  0  1 -1  0  0  0  0 -1  1;
    1 -1  0  0  0  0 -1  1  0  0  0  0;
    0  1  0  1 -1  0  0 -1  0 -1  1  0;
    0  0  1 -1  0  0  0  0 -1  1  0  0;
   -1  0  0 -1  0  0  1  0  0  1  0  0;
    0 -1 -1  0  0  0  0  1  1  0  0  0;
    0  0  0  0 -1  0  0  0  0  0  1  0;
    0  0  0  0  0 -1  0  0  0  0  0  1;
  
  ];
  
  % Mass balance equations
  dxdt = S * v;

