
%
% omm4.m
%
% Derives an expression for the steady-state of a variant
% of the Open Michaelis-Menten model, where the velocity of
% product formation is proportional to the square of the
% enzyme-substrate complex. Solving for this steady state
% requires a substitution strategy that results in a 
% superlinear term, and as with omm3, illustrates the
% process of using a pseudospecies to resolve the non-
% linearity.
%
% Optional Arguments:
% doUnsub - if true, the py-substitution is reversed prior
%       to returning vbar. Default value is true.
% verbose - if true, each calculation in the substitution
%       and solution of the steady state equation are 
%       printed to the console. Default value is false.
% fid - a file id to which to print text when verbose=true. 
%       Returned by a previous call to fopen(). Default 
%       value is 1, the console. This argument is ignored 
%       if verbose=false.
% wid - width of the device, in characters, to which to
%       print. Default value is 80.
%
% Value:
% vbar - the steady state vector of reaction velocities 
%        satisfying S * vbar = 0. Elements in vbar are 
%        either in terms of p and q (if doUnsub is false), 
%        or k and x (if doUnsub is true).
% psi  - the composite mapping function from the set 
%        union(K,X) to union(P,Q,R[P,Q]) if doUnsub is 
%        false, or to union(K_pq,X_pq) if doUnsub is true, 
%        satisfying psi(v) = vbar.

function [vbar,psi] = omm4( varargin )

  ip = inputParser;
  addParamValue(ip,'doUnsub',true,@islogical);
  addParamValue(ip,'verbose',false,@islogical);
  addParamValue(ip,'fid',1,@isnumeric);
  addParamValue(ip,'wid',80,@isnumeric);

  parse(ip,varargin{:});
  doUnsub = ip.Results.doUnsub;
  verbose = ip.Results.verbose;
  fid = ip.Results.fid;
  wid = ip.Results.wid;

  % Load the model description
  [S,v,k,x,xdot] = loadOMM();

  % Pretend the rate of product formation is proportional
  % to the square of the substrate-enzyme complex
  v(3) = k(3) * x(3)^2;
  xdot = S*v;

  if( verbose )
    printvar(sprintf(strcat('\nThere are %d reactions',...
      ' and %d species in the OMM model.\n'),...
      length(k), length(x)), 'fid', fid, 'wid', wid );
    printvar(sprintf(strcat('The stoichiometric matrix is S =\n')),...
      'fid', fid, 'wid', wid);
    printvar( S, 'vsym', 'S', 'fid', fid, 'wid', wid );
    printvar(sprintf(strcat('\nIn this variant of the OMM',...
      ' model, we assume the rate of product formation is',...
      ' proportional to the square of the',...
      ' enzyme-substrate complex:\n')),'fid',fid,'wid',wid);
    printvar( v(3), 'vsym', 'v(3)', 'vtype', 'scalar',...
      'wid', wid, 'fid', fid );
    printvar(sprintf(strcat('\nThe resulting vector of',...
      ' reaction velocities is v, where\n')),'fid',fid,'wid',wid);
    printvar( v, 'vsym', 'v', 'fid', fid, 'wid', wid );
    printvar(sprintf(strcat('\nThe vector of mass balance',...
      ' equations is xdot = S*v, giving\n')),'fid',fid,'wid',wid);
    printvar( xdot, 'vsym', 'xdot', 'fid', fid, 'wid', wid );
  end

  % Add the pseudospecies x5_hat=x3^2
  syms x5_hat positive real
  x = [ x; x5_hat ];
  v = subs(v,x(3)^2,x5_hat);
  
  if( verbose )
    printvar(sprintf(strcat('\nWe would like to',...
      ' define a map psi_p such that psi_p(x3) is in set Y.',...
      ' To do so we introduce a pseudospecies x5_hat = x3^2',...
      ' and let v(3) = k3*x5_hat. Now,\n')),'fid',fid,'wid',wid);
    printvar( v, 'vsym', 'v', 'fid', fid, 'wid', wid );
  end

  % Declare vectors of in/dependent variables
  p = sym( 'p', [5,1] ); y = sym( 'y', [5,1] );
  p = sym( p, 'positive' ); p = sym( p, 'real' );
  y = sym( y, 'positive' ); y = sym( y, 'real' );
  
  % Substitution strategy
  kx = [ k' x' ];
%  py = [ p(2:4)',y(4),p(5),y(1),p(1),y([2,3,5])' ];
  py = [ p(2:4)',y(5),p(5),y(4),p(1),y([1,3,2])' ];
 
  % Execute pysub
  [vbar,ybar,psi_py,q] = pysub( S, v, kx, py, y,...
    'verbose', false );

  % Make sure x1 (y4), k4 (y5) are pivots
  clear eqs
  yi = [4,5];
  a = sym( 'a', [2,1] ); a = sym( a, 'real' );
  ybar = subs( ybar, q, q.*a );

  for i = 1:length(yi)
    eqs{i} = (strcat(char(ybar(yi(i))),'=',char(q(i))));
  end
  avars = num2cell(a);
  asoln = solve( eqs{:}, avars{:} );
  qp = simplify( subs( q.*a, a, struct2cell(asoln) ) );
  [vbar,ybar,psi_py,qp] = pysub( S, v, kx, py, y, ...
    'q', qp, 'verbose', verbose, 'wid', wid, 'fid', fid );

  % Resolve the pseudospecies. Let x1 remain a pivot.
  q_hat = psi_py(find(psi_py==x(1)),2);
  lhs = psi_py(find(psi_py==x(3)),2);
  rhs = psi_py(find(psi_py==x5_hat),2);
  eq=strcat( '(', char(lhs), ')^2 = ', char(rhs) );
  q_soln = solve( eq, q_hat );
  q_soln = q_soln(1);
  vbar = simplify(subs(vbar,q_hat,q_soln));
  ybar = simplify(subs(ybar,q_hat,q_soln));
  psi_py_ok = simplify(subs(psi_py,q_hat,q_soln));

  if( verbose )
    printvar(sprintf(strcat('\nTo resolve the pseudospecies',...
      ' we require that psi_py(x5_hat) = psi_py(x3^2).',...
      ' In other words, %s. This gives\n'), eq),...
      'fid', fid, 'wid', wid);
    printvar( psi_py_ok, 'vtype', 'map', 'vsym', 'psi_py',...
      'fid', fid, 'wid', wid );
    printvar(sprintf('\nand\n'),'fid',fid,'wid',wid);
    printvar( vbar, 'vtype', 'vec', 'vsym', 'vbar',...
      'fid', fid, 'wid', wid );
  end

  % Reverse the substitution
  if( doUnsub )
    if( verbose )
      printvar(sprintf(strcat('\nWe may now proceed with',...
        ' the inverse substitution.')),'fid',fid,'wid',wid);
    end
    [vbar,psi_ss] = unsub( v,p,q,y,kx,py,ybar,psi_py_ok,...
      'verbose', verbose, 'fid', fid, 'wid', wid );
    psi = psi_ss;
  else
    psi = psi_py_ok;
  end
  
  % Verify steady state
  if( verbose )
    printvar(sprintf(strcat('\nVerify steady state.',...
      ' xdot = S * vbar, where\n')), 'fid', fid, 'wid', wid); 
    printvar( simplify(S*vbar), 'vtype', 'vec', 'vsym', 'xdot',...
      'fid', fid, 'wid', wid );
  end
