
%
% printvar.m
%
% Prints the given symbolic variable to the specified fileID.
%
% Required Arguments:
% var - the symbolic variable to be printed.
%
% Optional Arguments:
% vtype - the variable type. Must be in {'scalar','vec','matrix',
%         'map','string'}.
% vsym  - symbol used to represent elements of a vector or matrix.
%         Must be of type char.
% wid   - width of the device, in characters, to which to print.
%         Default value is 80.
% strop - a string representing the operator used when printing
%         out scalars or elements in a vector or matrix.
%         Default value is '='.
% dbspc - if true, elements in a vector, map, or matrix are double-
%         spaced. Default value is false.
% fid - the active file id to which to print var.
% 

function printvar( var, varargin )
  
  % Parse optional arguments
  validTypes = {'scalar','vec','matrix','map','string'};

  ip = inputParser;
  addParamValue(ip,'fid',1,@isnumeric);
  addParamValue(ip,'wid',80,@isnumeric);
  addParamValue(ip,'strop','=',@ischar);
  addParamValue(ip,'dbspc',false,@islogical);
  addParamValue(ip,'vtype','unknown',...
    @(x) any(validatestring(x,validTypes)));
  addParamValue(ip,'vsym','v',@ischar);
  parse(ip,varargin{:});

  fid = ip.Results.fid;
  winWidth = ip.Results.wid;
  vtype = ip.Results.vtype;
  vsym = ip.Results.vsym;
  strop = ip.Results.strop;

  if strcmp(vtype,'unknown')
    if isstr(var)
      vtype = 'string';
    elseif isscalar(var)
      vtype = 'scalar';
    elseif isvector(var)
      vtype = 'vec';
    else
      if size(var,2)==2
        vtype = 'map';
      else
        vtype = 'matrix';
      end
    end
  end

  % Print a simple string
  if strcmp(vtype,'string')

    slen = length(var);
    
    % Strings can span multiple lines
    i = 1;
    while i <= slen

      % Break at a space
      spaces = min(strfind(var(i:slen),' ')-winWidth,0);
      nextline = find(spaces==0);
      if ~isempty(nextline)
        j = i+spaces(nextline(1)-1)+winWidth-1;
      else
        j = slen;
      end

      fprintf( fid, '%s\n', var(i:j) );
      i = j+1;
    end
    
    if ip.Results.dbspc
      fprintf( fid, '\n' );
    end
  
  % Print a scalar
  elseif strcmp(vtype,'scalar')
    lhs = sprintf('  %s %s ',vsym, strop);
    rhs = char(var);
    
    lhslen = length(lhs);
    rhslen = length(rhs);
    
    % Scalars can span multiple lines
    i = 1;
    while i <= rhslen
      j = min(rhslen,i+winWidth-1-lhslen-1);
      fprintf( fid, '%s%s\n', lhs, rhs(i:j) );
      lhs = char(' '*ones(1,lhslen));
      i = j+1;
    end
    
    if ip.Results.dbspc
      fprintf( fid, '\n' );
    end
  
  % Print a mapping function
  elseif strcmp(vtype,'map')

    stropSpecified = false;
    for i = 1:length(varargin) stropSpecified = or(...
      stropSpecified,strcmp(char(varargin{i}),'strop')); end
    if ~stropSpecified
      strop = ' |--> ';
    end

    dgts = max(arrayfun(@(x) length(char(x)),var(:,1)));
    sfmt = sprintf('%%-%ds',dgts);

    % Max length of any element, in chars
    maxEleLen = max(arrayfun(@(x) length(char(x)), var(:,2)))+...
      length(sprintf('  %s %s ',sprintf(sfmt,char(var(1,1))), strop));
    maxLineLen = maxEleLen+length(char(var(1,1)));

    for i = 1:size(var,1)
      lhs = sprintf(sfmt,char(var(i,1)));
      printvar( var(i,2), 'vtype', 'scalar', 'vsym', lhs,...
        'fid', fid, 'wid', winWidth, 'dbspc', ...
        (maxLineLen>winWidth), 'strop', strop );
    end

  % Print a vector
  elseif strcmp(vtype,'vec')

    % Max length of any element, in chars
    if isnumeric(var)
      maxEleLen = max(arrayfun(@(x) length(int2str(x)), var));
    else
      maxEleLen = max(arrayfun(@(x) length(char(x)), var));
    end

    dgts = floor(log10(length(var)))+1;
    sfmt = sprintf('%%%dd',dgts);
    if length(strfind(vsym,'%s'))
      lhsfmt = vsym;
    else
      lhsfmt = sprintf('%s(%%s)',vsym);
    end

    maxLineLen = maxEleLen+length(...
      sprintf('%s(%s)',vsym,sprintf(sfmt,1)));

    for i = 1:length(var)
      lhs = sprintf(lhsfmt,sprintf(sfmt,i));
      printvar( var(i), 'vtype', 'scalar', 'vsym', lhs,...
        'fid', fid, 'wid', winWidth, 'dbspc', (maxLineLen>winWidth) );
    end
  
  % Print a matrix
  elseif strcmp(vtype,'matrix')

    % Length of each element, in chars
    if isnumeric(var)
      eleLens = arrayfun(@(x) length(int2str(x)), var);
    else
      eleLens = arrayfun(@(x) length(char(x)), var);
    end
    
    % Length of each column, in chars
    colLens = max(eleLens);
    
    % Length of a row, in chars
    rowLen  = sum(colLens)+2*length(colLens);
    
    % Length of a replaced element
    sfmt = sprintf('%%%dd,%%%dd',...
      floor(log10(size(var,1)))+1,floor(log10(size(var,2)))+1);
    sfmt = sprintf('%s',strcat(vsym,'(',sfmt,')'));
    eleEg = sprintf(sfmt,size(var,1),size(var,2));
    eleMax = length(eleEg);
    
    % Identify elements that could be replaced to shorten rowLen
    repSched = find( eleLens > eleMax );
    if ~isempty(repSched)
      [tmp,I] = sort( eleLens(repSched), 'descend' );
      repSched = repSched(I);
    end
    
    % Not small enough to print
    irep = 1;
    while (rowLen >= winWidth) & (irep <= length(repSched))
      eleLens(repSched(irep))=eleMax;
      colLens = max(eleLens);
      rowLen  = sum(colLens)+2*length(colLens);
      irep = irep + 1;
    end

    % Create the string version of var    
    Svar = cell(size(var,1),size(var,2));
    for i = 1:size(var,1)
      for j = 1:size(var,2)
        if isnumeric(var)
          Svar{i,j} = int2str(var(i,j));
        else 
          Svar{i,j} = char(var(i,j));
        end
      end
    end
    
    % Replace large entries
    if( irep > 1 )
      for i = irep-1:-1:1
        r = mod(repSched(i),size(var,1));
        c = floor(repSched(i)/size(var,1))+1;
        Svar{r,c} = sprintf(sfmt,r,c);
      end
    end
    
    % Print matrix
    for r = 1:size(var,1)
      for c = 1:size(var,2)
        elefmt = sprintf('%%%ds',colLens(c)+2);
        fprintf( fid, elefmt, Svar{r,c} );
      end
      fprintf( fid, '\n' );
    end
    
    % Print out-sized entries
    if( irep > 1 )
      fprintf( fid, '\nwhere\n\n' );
      for i = (irep-1):-1:1
        r = mod(repSched(i),size(var,1));
        c = floor(repSched(i)/size(var,1))+1;
        printvar( var(repSched(i)), 'vtype', 'scalar',...
          'vsym', sprintf(sfmt,r,c), 'fid', fid, ...
          'wid', winWidth, 'dbspc', true );
      end
    end
  
  % Unknown
  else
    fprintf(fid,'Error! Unknown symbolic variable type: ''%s''.\n',vtype);
    fprintf(fid,'Parameter vtype must be one of ''map'',''vec'', or ''matrix''\n\n.');

  end
