
#
# printvar.m
#
# Prints the given variable to the specified fileID.
#
# Required Arguments:
# var - the symbolic variable to be printed.
#
# Optional Arguments:
# vtype - the variable type. Must be one of SCALAR, VECTOR, MATRIX,
#         MAP, or STRING.
# vsym  - symbol used to represent elements of a vector or matrix.
#         Must be of type 'string'.
# wid   - width of the device, in characters, to which to print.
#         Default value is 80.
# strop - a string representing the operator used when printing
#         out scalars or elements in a vector or matrix.
#         Default value is '='.
# dbspc - if true, elements in a vector, map, or matrix are double-
#         spaced. Default value is false.
# fid - the active file id to which to print var.
# 

printvar := proc( var )
  
  local pvtype, pvsym, pstrop, pfid, pwid, pdbspc:
  local Options, Defaults:
  local lhs, rhs, lhslen, rhslen, slen:
  local dgts, sfmt, lhsfmt:
  local maxEleLen, maxLineLen:
  local i, j, r, c, irep:
  local nrows, ncols:
  local eleLens, colLens, rowLen, eleEg, eleMax:
  local repSched, maxReps:
  local Svar, elefmt:
  local rhsMax, doDbspc:

  Options := table():
  Defaults := table([
    OptionParms=table([
      'vtype'=[UNKNOWN,symbol],
      'vsym'=["v",string],
      'wid'=[80,integer],
      'strop'=["=",string],
      'dbspc'=[false,boolean],
      'fid'=[-1,integer]
    ]),PositionalParms=table()]):

  ProcessOptions( 1, [args], Defaults, Options );
  pvtype := Options[vtype]:
  pvsym := Options[vsym]:
  pstrop := Options[strop]:
  pwid := Options[wid]:
  pdbspc := Options[dbspc]:

  if Options[fid] < 0 then
    pfid := terminal:
  else
    pfid := Options[fid]:
  end if:


  # Process optional parameters  
  if pvtype=UNKNOWN then
    if type(var,'string') then
      pvtype := STRING:
    elif type(var,'Matrix') then
      if LinearAlgebra[Dimension](var)[2]=2 then
        pvtype := MAP:
      else
        pvtype := MATRIX:
      end if
    elif type(var,'Vector') then
      pvtype := VECTOR:
    else
      pvtype := SCALAR:
    end if;
  else
    if not member(pvtype,{SCALAR,VECTOR,MATRIX,MAP,STRING}) then
      ERROR("Optional parameter 'vtype' must be one of",
        "SCALAR, VEC, MATRIX, or MAP."):
    end if
  end if:

  # Print a string
  if pvtype=STRING then
    slen := length(var):
    
    # Strings can span multiple lines
    i := 1:
    while i <= slen do

      # Only break at a space
      j := i + pwid - 1:
      if j > slen then
        j := slen:
      else
        j := i + StringTools[FirstFromRight](" ",
          StringTools[SubString](var,i..j))-1:
      end if:
      fprintf( pfid, "%s\n", StringTools[SubString](var,i..j) ):
      i := j+1:
    end do:
    
    if pdbspc then
      fprintf( pfid, "\n" ):
    end if:
  
  # Print a scalar
  elif pvtype=SCALAR then
    lhs := sprintf("  %s %s ",pvsym, pstrop):
    rhs := convert( var, 'string' ):
    rhs := StringTools[SubstituteAll](rhs,"+"," + "):
    rhs := StringTools[SubstituteAll](rhs,"]-","] - "):
    rhs := StringTools[SubstituteAll](rhs,"[",""):
    rhs := StringTools[SubstituteAll](rhs,"]",""):
    
    lhslen := StringTools[Length](lhs);
    rhslen := StringTools[Length](rhs);

    # Scalars can span multiple lines
    i := 1:
    while i <= rhslen do
      j := min(rhslen,i+pwid-1-lhslen-1):
      fprintf( pfid, "%s%s\n", lhs, rhs[i..j] ):
      lhs := StringTools[Repeat](" ",lhslen):
      i := j+1;
    end do:
    
    if pdbspc then
      fprintf( pfid, "\n" ):
    end if
  
  # Print a mapping function
  elif pvtype=MAP then

    # Max length of any element, in chars
    maxEleLen := max(map(proc(x) StringTools[Length](convert(x,'string')) end proc, 
      var[..,2]) ):

    dgts := max(map(proc(x) StringTools[Length](convert(x,'string')) end proc, var[..,1]) )-2:
    sfmt := sprintf("%%-%ds",dgts):

    for i from 1 to LinearAlgebra[Dimension](var)[1] do
      lhs := convert(var[i,1],'string'):
      lhs := StringTools[SubstituteAll](lhs,"[",""):
      lhs := StringTools[SubstituteAll](lhs,"]",""):
      lhs := sprintf(sfmt,lhs):
      if i = 1 then
        maxLineLen := maxEleLen+StringTools[Length](
          sprintf("  %s %s ",lhs, pstrop))
      end if:
      printvar( var[i,2], vtype=SCALAR, vsym=lhs, fid=Options[fid], 
        wid=pwid, dbspc=evalb(maxLineLen>pwid), strop=" |--> " ):
    end do:

  # Print a vector
  elif pvtype=VECTOR then

    # Max length of any element, in chars
    maxEleLen := max(map(proc(x) StringTools[Length](convert(x,'string')) end proc,
      var) ):

    dgts := floor(log10(LinearAlgebra[Dimension](var)))+1:
    sfmt := sprintf("%%%dd",dgts):
    if StringTools[Search]("%s",pvsym) > 0 then
      lhsfmt := pvsym:
    else
      lhsfmt := sprintf("%s[%%s]",pvsym):
    end if:

    maxLineLen := maxEleLen+StringTools[Length](
      sprintf("%s[%s]",pvsym,sprintf(sfmt,1))):

    for i from 1 to LinearAlgebra[Dimension](var) do
      lhs := sprintf(lhsfmt,sprintf(sfmt,i)):
      printvar( var[i], vtype=SCALAR, vsym=lhs, fid=Options[fid],
        wid=pwid, dbspc=evalb(maxLineLen>pwid) ):
    end do:
  
  # Print a matrix
  elif pvtype=MATRIX then

    nrows := LinearAlgebra[Dimension](var)[1]:
    ncols := LinearAlgebra[Dimension](var)[2]:

    # Length of each element, in chars
    eleLens:= map(proc(x) 
      local eleStr:
      eleStr := (convert(x,'string')):
      eleStr := StringTools[SubstituteAll](eleStr,"[",""):
      eleStr := StringTools[SubstituteAll](eleStr,"]",""):
      return(StringTools[Length](eleStr))
      end proc,
      var ):
    rhsMax := max(eleLens):

    # Length of each column, in chars
    colLens := Vector[row](ncols):
    for i from 1 to ncols do colLens[i] := max(eleLens[..,i]) end do:
    
    # Length of a row, in chars
    rowLen := add(i, i in colLens) + 2*ncols:
    
    # Length of a replaced element
    sfmt := sprintf("%%%dd,%%%dd",
      floor(log10(nrows))+1,floor(log10(ncols))+1):
    sfmt := sprintf("%s",cat(pvsym,"[",sfmt,"]")):
    eleEg := sprintf(sfmt,nrows,ncols):
    eleMax := StringTools[Length](eleEg):

    # Identify elements that could be replaced to shorten rowLen
    repSched := Matrix(1,3,0):
    for i from 1 to nrows do for j from 1 to ncols do
      if eleLens[i,j] > eleMax then 
        repSched := ArrayTools[Concatenate](1,repSched,
          Matrix(1,3,[eleLens[i,j],i,j])):
      end if:
    end do: end do:
    repSched := Matrix(sort(convert(repSched,'listlist'))):

    # Not small enough to print
    maxReps := LinearAlgebra[Dimension](repSched)[1]:
    irep := maxReps:
    while (rowLen >= pwid) and (irep > 1) do
      eleLens[repSched[irep,2],repSched[irep,3]] := eleMax:
      for j from 1 to ncols do colLens[j] := max(eleLens[..,j]) end do:
      rowLen := add(j, j in colLens) + 2*ncols:
      irep := irep - 1:
    end do:

    # Still too large to print. Print out non-zero elements only.
    if rowLen > pwid then
      
      doDbspc := false:
      if rhsMax >= pwid then
        doDbspc := true:
      end if:
      
      for r from 1 to nrows do
        for c from 1 to ncols do
          if var[r,c] <> 0 then
            lhs := sprintf(sfmt,r,c):
            printvar( var[r,c], vtype=SCALAR, vsym=lhs,
              fid=Options[fid], wid=pwid, dbspc=doDbspc ):
          end if:
        end do:
      end do:
      printvar( cat("\nAnd all other elements in ",
        pvsym, " are zero."), fid=Options[fid], wid=pwid )
    
    # Matrix fit in width
    else
    
      # Create the string version of var    
      Svar := map(proc(x) 
        local eleStr:
        eleStr := convert(x,string):
        eleStr := StringTools[SubstituteAll](eleStr,"[",""):
        eleStr := StringTools[SubstituteAll](eleStr,"]",""):
        end proc, var):
      
      # Replace large entries
      if irep < maxReps then
        for i from irep+1 to maxReps do
          r := repSched[i,2]:
          c := repSched[i,3]:
          Svar[r,c] := sprintf(sfmt,r,c):
        end do:
      end if:
      
      # Print matrix
      for r from 1 to nrows do
        for c from 1 to ncols do
          elefmt := sprintf("%%%ds",colLens[c]+2):
          fprintf( pfid, elefmt, Svar[r,c] ):
        end do:
        fprintf( pfid, "\n" );
      end do:
      
      # Print out-sized entries
      if irep < maxReps then
        fprintf( pfid, "\nwhere\n\n" ):
        for i from irep+1 to maxReps do
          r := repSched[i,2]:
          c := repSched[i,3]:
          printvar( var[r,c], vtype=SCALAR, vsym=sprintf(sfmt,r,c), 
            fid=Options[fid], wid=pwid, dbspc=true ):
        end do:
      end if:
    end if:
  
  # Unknown
  else
    fprintf(pfid,"Error! Unknown symbolic variable type: \"%s\".\n",pvtype);
    fprintf(pfid,"Parameter vtype must be one of SCALAR, VECTOR, MATRIX, or MAP.\n\n");

  end if:

end proc: