
%
% pysub.m
%
% Solves,
%
%     S * v = 0
%
% where v is a vector of mass action reaction velocities described
% in terms of elements in the vector kx. Returns a mapping function
% psi_py and vector vbar where
%
%     vbar = psi_py( v )
%
% and
%
%     S * vbar = 0.
% 
% The vector ybar is returned to facilitate alternative steady
% state expressions. Note that
%
%     ybar = N * q
%
% where N is the matrix whose column vectors form a basis for the
% coefficient matrix C = S * P, where P is the jacobian of the
% linearized velocity vector v = P * y.
%
% Required Arguments:
% S  - a stoichiometric matrix. Element s_{i,j} gives the 
%      stoichiometry of species i in reaction j.
% v  - a vector of biochemical reaction velocities
% kx - a concatenated vector of symbols representing rate constants
%      and species concentrations appearing in v
% py - defines a map psi_p : kx(i) |--> py(i) such that v is linear
%      in y
% y  - a symbolic vector of dependent variables
%
% Optional Arguments:
% q   - a vector of null space basis vector coefficients.
% verbose - if true, each calculation in the substitution and
%       solution of the steady state equation are printed to the 
%       console. Default value is false.
% fid - a file id to which to print text when verbose=true. Returned
%       by a previous call to fopen(). Default value is 1, the console. 
%       This argument is ignored if verbose=false.
% wid - width of the device, in characters, to which to print.
%       Default value is 80.
% 
% Value:
% vbar - a steady state vector of reaction velocities, S * vbar = 0.
% ybar - a linear combination of null space basis vectors. See above.
% psi_py - a two-column array where psi_py(i,2) is the steady state
%        expression for model element psi_py(i,1).
% q    - the vector of null space basis vector coefficients,
%        ybar = N * q
%

function [vbar,ybar,psi_py,q] = pysub( S, v, kx, py, y, varargin )

  % Parse optional arguments
  ip = inputParser;
  addParamValue(ip,'verbose',false,@islogical);
  addParamValue(ip,'fid',1,@isnumeric);
  addParamValue(ip,'wid',80,@isnumeric);
  addParamValue(ip,'q',sym('q',0));
  parse(ip,varargin{:});
  verbose = ip.Results.verbose;
  fid = ip.Results.fid;
  wid = ip.Results.wid;
  q = ip.Results.q;


  %
  % Execute
  %
  
  % Primary map
  psi_p = [ kx' py' ];
  if( verbose )
    printvar(sprintf('\nLet the map psi_p be given by\n'),...
      'fid', fid, 'wid', wid); 
    printvar( psi_p,'fid',fid,'wid',wid );
  end
  
  % Linear velocity vector
  vpy = subs( v, kx, py );
  if( verbose )
    printvar(sprintf(strcat('\nThis results in a linear velocity',...
      ' vector psi_p(v), where\n')), 'fid', fid, 'wid', wid);
    printvar( vpy, 'vsym', 'psi_p(v(%s))', 'fid', fid, 'wid', wid );
  end

  % Calculate coefficient matrix
  P = jacobian( vpy, y );
  C = S*P;
  if( verbose )
    y_f = findsym(y);
    if length(strfind(y_f,',')>0)
      tmp = strfind(y_f,',');
      y_f = y_f((tmp(length(tmp))+1):length(y_f));
    end
    printvar(sprintf(strcat('\nWe can express psi_p(v) as',...
      ' the product P*y, where y is the vector [y1,...,%s]^T',...
      ' and P =\n'), y_f), 'fid', fid, 'wid', wid);
    printvar( P, 'vtype', 'matrix', 'vsym', 'P', 'fid', fid, 'wid', wid );
    printvar(sprintf(strcat('\nFrom this we calculate the',...
      ' coefficient matrix, C = S*P =\n')), 'fid', fid, 'wid', wid);
    printvar( C, 'vtype', 'matrix', 'vsym', 'C', 'fid', fid, 'wid', wid );
  end

  % Row-reduce the coefficient matrix
  Crref = rref(C);
  if( verbose )
    printvar(sprintf(strcat('\nC is row equivalent to the',...
      ' reduced matrix Crref =\n')),'fid',fid,'wid',wid);
    printvar( Crref, 'vsym', 'Crref', 'fid', fid, 'wid', wid );
  end
  
  % Derive a basis for the null space
  N = null(C);
  if( verbose )
    printvar(sprintf(strcat('\nThe null space of C is',...
      ' spanned by the columns of N =\n')),'fid',fid,'wid',wid);
    printvar( N, 'vsym', 'N', 'vtype', 'matrix', ...
      'fid', fid, 'wid', wid );
  end
  
  % Solve for y
  default_q = sym( 'q', [size(N,2),1] );
  default_q = sym( default_q, 'real' );
  if isempty(q)
    q = default_q;
  end

  ybar = simplify(N*q);
  if( verbose )
    printvar(sprintf('\nLet ybar = N*q, where q is given by\n'),...
      'fid', fid, 'wid', wid );
    printvar( q, 'vtype', 'vec', 'vsym', 'q', ...
      'fid', fid, 'wid', wid );
    printvar(sprintf('\nThis gives\n'),'fid',fid,'wid',wid);
    printvar( ybar, 'vsym', 'ybar', 'fid', fid, 'wid', wid );
  end

  % Composite fwd map psi_py
  psi_py = [ kx' subs(py',y,ybar) ];
  if( verbose )
    printvar(sprintf(strcat('\nFrom ybar we construct the',...
      ' composite forward map psi_py : \n')),'fid',fid,'wid',wid);
    printvar( psi_py, 'fid', fid, 'wid', wid );
  end
  
  % Steady state velocity vector
  vbar = subs(v,psi_py(:,1),psi_py(:,2));
  if( verbose )
    printvar(sprintf(strcat('\nThe steady state reaction',...
      ' velocity vector vbar is given by psi_py(v), where\n')),...
      'fid',fid,'wid',wid);
    printvar( vbar, 'vsym', 'vbar', 'fid', fid, 'wid', wid );
  end

  % Always return the default q
  q = default_q;
