
%
% unsub.m
%
% Generates and returns the composite steady state map
% psi_ss = ( psi_{qp}^{-1} \dot psi_{py} ) where
%
%     vbar = psi_ss( v )
%
% and
%
%     S * vbar = 0.
%
% Call unsub() after a call to pysub() to map steady
% state expressions back to elements in kx.
%
% Arguments:
% v  - the original vector of reaction velocities, with
%      elements defined in terms of k and x
% p  - a vector of independent parameters
% q  - a vector of basis vector coefficients, as returned
%      by pysub()
% y  - a vector of dependent parameters
% kx - the concatentated vector of rate constants and
%      species abundances
% py - the vector py used in a preceding call to pysub()
% ybar - the vector of dependent parameters that results
%      from solving the steady state equation, as returned
%      by pysub()
% psi_py - the composite forward map that linearizes the
%      vector v and solves the steady state equation.
%      Returned by pysub().
%
% Optional Arguments:
% verbose - if true, each calculation in the substitution and
%      solution of the steady state equation are printed to the 
%      console. Default value is false.
% fid - a file id to which to print text when verbose=true. Returned
%      by a previous call to fopen(). Default value is 1, the console. 
%      This argument is ignored if verbose=false.
% wid   - width of the device, in characters, to which to print.
%         Default value is 80.
% 
% 
% The values returned by unsub() are:
%
% vbar - the reaction velocity vector vbar=psi_ss(v),
%        where S * vbar = 0
% psi_ss - the composite forward and backward map that 
%        linearizes the vector v via the forward map
%        psi_py, solves the steady state equation, then
%        reverses the substitution.
%

function [vbar,psi_ss] = unsub( v, p, q, y, kx, py, ybar, psi_py, varargin )

  % Parse optional arguments
  ip = inputParser;
  addParamValue(ip,'verbose',true,@islogical);
  addParamValue(ip,'fid',1,@isnumeric);
  addParamValue(ip,'wid',80,@isnumeric);

  parse(ip,varargin{:});
  verbose = ip.Results.verbose;
  fid = ip.Results.fid;
  wid = ip.Results.wid;

  % Check q
  vqvector = findsym(q);
  vqindets = findsym(sym('q',[length(q),1]));
  if ~strcmp(vqvector,vqindets)
    error(strcat('Argument q must be the vector where q(i)=q_i,',...
      ' i = 1..dq, as returned by pysub().'));
  end

  % The mapping function psi_p
  psi_p = [ kx' py' ];

  % Derive the maping function psi_q
  [q,iq,iy] = intersect( q, ybar );
  psi_q = [ ybar(iy), y(iy) ];
  if( verbose )
    printvar(sprintf('\nThe mapping function psi_q^-1 is given by\n'),...
      'fid',fid,'wid',wid);
    printvar( psi_q,'vtype','map','fid',fid,'wid',wid );
  end
  
  % Composite backward map
  qp = [ p' q' ];
  psi_qp = [ qp' subs(subs(qp',psi_q(:,1),psi_q(:,2)),...
    psi_p(:,2),psi_p(:,1)) ];
  if( verbose )
    printvar(sprintf('\nThe composite inverse map psi_qp^-1:\n'),...
      'fid',fid,'wid',wid);
    printvar( psi_qp, 'fid', fid, 'wid', wid );
  end
  
  % Final steady state map
  psi_ss = [ kx' subs(subs(kx',psi_py(:,1),psi_py(:,2)),...
    psi_qp(:,1),psi_qp(:,2)) ];
  if( verbose )
    printvar(sprintf('\nThe complete steady state map psi_ss is therefore\n'),...
      'fid',fid,'wid',wid);
    printvar( psi_ss, 'fid', fid, 'wid', wid );
  end
  
  % Steady state velocity vector
  vbar = subs(v,psi_ss(:,1),psi_ss(:,2));
  
  if( verbose )
    printvar(sprintf(strcat('\nThe unsubstituted steady state reaction',...
      ' velocity vector vbar = psi_ss(v) is given by\n')),'fid',fid,'wid',wid);
    printvar( vbar, 'vsym', 'vbar', 'fid', fid, 'wid', wid );
  end

