
# 
# xearm.mpl
# 
# Returns a steady state expression for the xEARM model.
# 
# Optional Arguments:
# doUnsub - if true, the py-substitution is reversed
#      prior to returning vbar. Default value is true.
# verbose - if true, each calculation in the substitution
#      and solution of the steady state equation are
#      printed to the console. Default value is false.
# fid - the active file id to which to print var.
# wid - width of the device, in characters, to which to
#       print. Default value is 80.
#
# Value:
# vbar - the reaction velocity vector psi(v) where
#        S * vbar = 0
# psi  - the steady state mapping function psi(v)=vbar.
#        Depending on the value of doUnsub, this will
#        either be the forward mapping function psi_py
#        or the composite function psi_ss.
#

xearm := proc()

  local S, vv, vk, vx, vc:
  local nk, nx, nc, ns, np, ny:
  local vp, vy, py, vq:
  local PI, psi, psi_py:
  local p, y, k, x, c:
  local vbar, ybar, kxbar, kx, xdot:
  local Options, Defaults:
  local pverb, punsub, pfid, pwid:
  local q1soln, q3soln, q1poly, q3poly:
  local degree_q1poly, degree_q3poly:
  local avals, acoef, bvals, bcoef:

  read "printvar.mpl":
  read "pysub.mpl":

  # Parse out optional parameters
  Options := table():
  Defaults := table([
    OptionParms=table([
      'doUnsub'=[true,boolean],
      'verbose'=[false,boolean],
      'wid'=[80,integer],
      'fid'=[-1,integer]
    ]),PositionalParms=table()]):

  ProcessOptions( 0, [args], Defaults, Options );
  pverb := Options[verbose]:
  punsub := Options[doUnsub]:
  pwid := Options[wid]:
  pfid := Options[fid]:

  # Load the model description
  read "loadXearm.mpl":
  read "pysub.mpl":
  (S,vv,vk,vx,vc) := loadXearm():

  # System dimensions
  nk := LinearAlgebra[Dimension](vk):
  nx := LinearAlgebra[Dimension](vx):
  nc := LinearAlgebra[Dimension](vc):

  # Print out model and substitution strategy
  if pverb then
    printvar(sprintf("\nThere are %d reactions and %d"
      " species in the xEARM model.\n",nk,nx),fid=pfid,wid=pwid):
    printvar("The stoichiometric matrix is S, where\n", 
      fid=pfid, wid=pwid ):
    printvar( S, vsym="S", fid=pfid, wid=pwid ):
    printvar(sprintf("\nThe vector of reaction"
      " velocities is v, where\n"),fid=pfid,wid=pwid):
    printvar( vv, vsym="v", fid=pfid, wid=pwid ):
    printvar(sprintf("\nThe vector of mass balance"
      " equations is xdot = S*v, where \n"),fid=pfid,wid=pwid):
    printvar( LinearAlgebra[Multiply](S,vv),
      vsym="xdot", fid=pfid, wid=pwid ):
  end if:
  

  # Introduce pseudospecies
  #
  # Let vx[59] = x[32]^2
  # Let vx[60] = x[35]^2
  vx := ArrayTools[Concatenate](1,vx,x[59],x[60]):
  vv[38] := k[38]*c[1]*x[59]:
  vv[42] := k[42]*c[1]*x[60]:
  
  if pverb then
    printvar( "\nTo solve for xdot=0, we will introduce"
      "the following pseudospecies:\n", fid=pfid, wid=pwid ):
    printvar( "\n  x[59] = x[32]^2", fid=pfid, wid=pwid ):
    printvar( "  x[60] = x[35]^2", fid=pfid, wid=pwid ):
    printvar( "\n\nThis gives a new reaction velocity vector v, where\n",
      fid=pfid, wid=pwid ):
    printvar( vv[[38,42]], vtype=VECTOR, vsym="v",
      fid=pfid, wid=pwid ):
  end if:
  

  # New system dimensions
  nx := LinearAlgebra[Dimension](vx):

  # Declare vectors of in/dependent variables
  ns := nx + nk + nc:
  np := 104:
  ny := ns - np:

  vp := Vector( np, symbol=p ):
  vy := Vector( ny, symbol=y ):

  kx := ArrayTools[Concatenate]( 1, vk, vx, vc ):
  py := ArrayTools[Concatenate]( 1, vp, vy ):

 
  #
  # Py-substitution strategy
  #
  # PI_ij = 1 if psi_p( kx_j ) = py_i
  #

  PI := Matrix( ns ):
  p := proc(i) return i   : end proc:
  y := proc(i) return i+np: end proc:
  k := proc(i) return i   : end proc:
  x := proc(i) return i+nk: end proc:
  c := proc(i) return i+nk+nx: end proc:

  PI( p(1),   k(1))   := 1:
  PI( p(2),   k(2))   := 1:
  PI( p(3),   k(3))   := 1:
  PI( p(4),   k(4))   := 1:
  PI( p(5),   k(5))   := 1:
  PI( p(6),   k(6))   := 1:
  PI( p(7),   k(7))   := 1:
  PI( p(8),   k(8))   := 1:
  PI( p(9),   k(9))   := 1:
  PI( p(10),  k(10))  := 1:
  PI( p(11),  k(11))  := 1:
  PI( p(12),  k(12))  := 1:
  PI( p(13),  k(13))  := 1:
  PI( p(14),  k(14))  := 1:
  PI( p(15),  k(15))  := 1:
  PI( p(16),  k(16))  := 1:
  PI( p(17),  k(17))  := 1:
  PI( p(18),  k(18))  := 1:
  PI( p(19),  k(19))  := 1:
  PI( p(20),  k(20))  := 1:
  PI( p(21),  k(21))  := 1:
  PI( p(22),  k(22))  := 1:
  PI( p(23),  k(23))  := 1:
  PI( p(24),  k(24))  := 1:
  PI( p(25),  k(25))  := 1:
  PI( p(26),  k(26))  := 1:
  PI( p(27),  k(27))  := 1:
  PI( p(28),  k(28))  := 1:
  PI( p(29),  k(29))  := 1:
  PI( p(30),  k(30))  := 1:
  PI( p(31),  k(31))  := 1:
  PI( p(32),  k(32))  := 1:
  PI( p(33),  k(33))  := 1:
  PI( p(34),  k(34))  := 1:
  PI( p(35),  k(35))  := 1:
  PI( p(36),  k(36))  := 1:
  PI( p(37),  k(37))  := 1:
  PI( p(38),  k(38))  := 1:
  PI( p(39),  k(39))  := 1:
  PI( p(40),  k(40))  := 1:
  PI( p(41),  k(41))  := 1:
  PI( p(42),  k(42))  := 1:
  PI( p(43),  k(43))  := 1:
  PI( p(44),  k(44))  := 1:
  PI( p(45),  k(45))  := 1:
  PI( p(46),  k(46))  := 1:
  PI( p(47),  k(47))  := 1:
  PI( p(48),  k(48))  := 1:
  PI( p(49),  k(49))  := 1:
  PI( p(50),  k(50))  := 1:
  PI( p(51),  k(51))  := 1:
  PI( p(52),  k(52))  := 1:
  PI( p(53),  k(53))  := 1:
  PI( p(54),  k(54))  := 1:
  PI( p(55),  k(55))  := 1:
  PI( p(56),  k(56))  := 1:
  PI( p(57),  k(57))  := 1:
  PI( p(58),  k(58))  := 1:
  PI( p(59),  k(59))  := 1:
  PI( p(60),  k(60))  := 1:
  PI( p(61),  k(61))  := 1:
  PI( p(62),  k(62))  := 1:
  PI( p(63),  k(63))  := 1:
  PI( p(64),  k(64))  := 1:
  PI( p(65),  k(65))  := 1:
  PI( p(66),  k(66))  := 1:
  PI( p(67),  k(67))  := 1:
  PI( p(68),  k(68))  := 1:
  PI( p(69),  k(69))  := 1:
  PI( p(70),  k(70))  := 1:
  PI( p(71),  k(75))  := 1:
  PI( p(72),  k(80))  := 1:
  PI( p(73),  k(85))  := 1:
  PI( p(74),  k(90))  := 1:
  PI( p(75),  k(91))  := 1:
  PI( p(76),  k(92))  := 1:
  PI( p(77),  k(93))  := 1:
  PI( p(78),  k(94))  := 1:
  PI( p(79),  k(99))  := 1:
  PI( p(80),  k(100)) := 1:
  PI( p(81),  k(103)) := 1:
  PI( p(82),  k(106)) := 1:
  PI( p(83),  k(107)) := 1:
  PI( p(84),  k(110)) := 1:
  PI( p(85),  k(112)) := 1:
  PI( p(86),  k(115)) := 1:
  PI( p(87),  x(2))   := 1:
  PI( p(88),  x(5))   := 1:
  PI( p(89),  x(7))   := 1:
  PI( p(90),  x(10))  := 1:
  PI( p(91),  x(12))  := 1:
  PI( p(92),  x(15))  := 1:
  PI( p(93),  x(19))  := 1:
  PI( p(94),  x(21))  := 1:
  PI( p(95),  x(24))  := 1:
  PI( p(96),  x(27))  := 1:
  PI( p(97),  x(29))  := 1:
  PI( p(98),  x(33))  := 1:
  PI( p(99),  x(39))  := 1:
  PI( p(100), x(42))  := 1:
  PI( p(101), x(45))  := 1:
  PI( p(102), x(49))  := 1:
  PI( p(103), x(52))  := 1:
  PI( p(104), c(1))   := 1:
  PI( y(1),   x(1))   := 1:
  PI( y(2),   x(3))   := 1:
  PI( y(3),   x(4))   := 1:
  PI( y(4),   x(6))   := 1:
  PI( y(5),   x(8))   := 1:
  PI( y(6),   x(9))   := 1:
  PI( y(7),   x(11))  := 1:
  PI( y(8),   x(13))  := 1:
  PI( y(9),   x(14))  := 1:
  PI( y(10),  x(16))  := 1:
  PI( y(11),  x(17))  := 1:
  PI( y(12),  x(18))  := 1:
  PI( y(13),  x(20))  := 1:
  PI( y(14),  x(22))  := 1:
  PI( y(15),  x(23))  := 1:
  PI( y(16),  x(25))  := 1:
  PI( y(17),  x(26))  := 1:
  PI( y(18),  x(28))  := 1:
  PI( y(19),  x(30))  := 1:
  PI( y(20),  x(31))  := 1:
  PI( y(21),  x(32))  := 1:
  PI( y(22),  x(34))  := 1:
  PI( y(23),  x(35))  := 1:
  PI( y(24),  x(36))  := 1:
  PI( y(25),  x(37))  := 1:
  PI( y(26),  x(38))  := 1:
  PI( y(27),  x(40))  := 1:
  PI( y(28),  x(41))  := 1:
  PI( y(29),  x(43))  := 1:
  PI( y(30),  x(44))  := 1:
  PI( y(31),  x(46))  := 1:
  PI( y(32),  x(47))  := 1:
  PI( y(33),  x(48))  := 1:
  PI( y(34),  x(50))  := 1:
  PI( y(35),  x(51))  := 1:
  PI( y(36),  x(53))  := 1:
  PI( y(37),  x(54))  := 1:
  PI( y(38),  x(55))  := 1:
  PI( y(39),  x(56))  := 1:
  PI( y(40),  x(57))  := 1:
  PI( y(41),  x(58))  := 1:
  PI( y(42),  x(59))  := 1:
  PI( y(43),  x(60))  := 1:
  PI( y(44),  k(71))  := 1:
  PI( y(45),  k(72))  := 1:
  PI( y(46),  k(73))  := 1:
  PI( y(47),  k(74))  := 1:
  PI( y(48),  k(76))  := 1:
  PI( y(49),  k(77))  := 1:
  PI( y(50),  k(78))  := 1:
  PI( y(51),  k(79))  := 1:
  PI( y(52),  k(81))  := 1:
  PI( y(53),  k(82))  := 1:
  PI( y(54),  k(83))  := 1:
  PI( y(55),  k(84))  := 1:
  PI( y(56),  k(86))  := 1:
  PI( y(57),  k(87))  := 1:
  PI( y(58),  k(88))  := 1:
  PI( y(59),  k(89))  := 1:
  PI( y(60),  k(95))  := 1:
  PI( y(61),  k(96))  := 1:
  PI( y(62),  k(97))  := 1:
  PI( y(63),  k(98))  := 1:
  PI( y(64),  k(101)) := 1:
  PI( y(65),  k(102)) := 1:
  PI( y(66),  k(104)) := 1:
  PI( y(67),  k(105)) := 1:
  PI( y(68),  k(108)) := 1:
  PI( y(69),  k(109)) := 1:
  PI( y(70),  k(111)) := 1:
  PI( y(71),  k(113)) := 1:
  PI( y(72),  k(114)) := 1:


  # Py-sub
  vbar,ybar,psi,vq := Pysub( S, vv, kx, py, vy, PI,
    doCrref=false, verbose=pverb, wid=pwid, fid=pfid ):
  if pverb then
    printvar( "\nAfter solving psi_p(xdot)=0, we have"
      " the following composite forward map psi_py,\n",
      fid=pfid, wid=pwid ):
    printvar( psi, vtype=MAP, fid=pfid, wid=pwid ):
  end if:
  
  # Resolve the pseudospecies. Note that entering
  #
  # > q2soln,q3soln := solve( {
  #     ybar[43]=ybar[23]^2,
  #     ybar[42]=ybar[21]^2
  # }, {'q[2]','q[3]'} ):
  #
  # breaks my machine. Instead, solve for q[3] first,
  # create a single fourth-order polynomial in q[2],
  # substitute out the coefficients, solve, then
  # substitute back in on subsequent call to Pysub().
  
  # (Use PolynomialTools instead, see next line)
  # q3soln_full := solve( ybar[43]=ybar[23]^2, q[3] ):
  # q3soln := simplify( b3*q3soln_full/coeff(q3soln_full,q[3]^2) ):
  
  q3poly := collect(simplify(ybar[43]-ybar[23]^2),'q[3]'):
  bvals := PolynomialTools[CoefficientVector](q3poly,q[3]):
  degree_q3poly := LinearAlgebra[Dimension](bvals):
  bcoef := Vector(degree_q3poly, symbol='b'):
  q3poly := PolynomialTools[FromCoefficientVector](
    bcoef, 'q3' ):
  q3soln := solve( q3poly, 'q3' ):

  q1poly := collect(subs(q[3]=subs(Equate(bcoef,map(factor,bvals)),
    q3soln),ybar[42]-ybar[21]^2),'q[1]'):
  avals := PolynomialTools[CoefficientVector](q1poly,q[1]):
  degree_q1poly := LinearAlgebra[Dimension](avals):
  acoef := Vector(degree_q1poly, symbol='a'):
  q1poly := PolynomialTools[FromCoefficientVector](
    acoef, 'q1' ):
  #for i from 1 to 5 do
  #  map((x,y) -> if (evalb(y=0)) then x:=0 fi,
  #    acoef[i], avals[i]):
  #end do:
  q1soln := solve(q1poly=0,'q1'):
  q1soln := max(q1soln[1],q1soln[2]) assuming 
    a[1]>0,a[2]>0,a[3]>0,a[2]^2>4*a[1]*a[3];  

  if pverb then
    printvar( "\nTo resolve the pseudospecies, we requre that\n",
      fid=pfid, wid=pwid ):
    printvar( "  psi_py(x[59]) = psi_py(x[32]^2) and",
      fid=pfid, wid=pwid ):
    printvar( "  psi_py(x[60]) = psi_py(x[35]^2).",
      fid=pfid, wid=pwid ):
    printvar( "\nIn other words, we require that\n",
      fid=pfid, wid=pwid ):
    printvar( cat("  ybar[42] = ybar[21]^2 and",
      StringTools[Repeat](" ",pwid-32), "(1)"),
      fid=pfid, wid=pwid ):
    printvar( cat("  ybar[43] = ybar[23]^2.",
      StringTools[Repeat](" ",pwid-29), "(2)\n"),
      fid=pfid, wid=pwid ):
    printvar( "\nTo solve these two equations, note that\n",
      fid=pfid, wid=pwid ):
    printvar( "  > indets(ybar[23]) intersect convert(vq,set) = {q[2]}",
      fid=pfid, wid=pwid ):
    printvar( "  > indets(ybar[43]) intersect convert(vq,set) = {q[3]}",
      fid=pfid, wid=pwid ):
    printvar( "  > indets(ybar[21]) intersect convert(vq,set) = {q[1],q[2],q[3]}",
      fid=pfid, wid=pwid ):
    printvar( "  > indets(ybar[42]) intersect convert(vq,set) = {q[2],q[3]}",
      fid=pfid, wid=pwid ):
    printvar( "\nIn other words, we can first solve"
      " Equation (1) for q[3], then use that solution to"
      " solve Equation (2) for q[1]. The solution to Equation(1) is given by\n", 
      fid=pfid, wid=pwid):
    printvar( q3soln, vtype=SCALAR, vsym="q[3]",
      fid=pfid, wid=pwid ):
    printvar( "\nwhere\n", fid=pfid, wid=pwid ):
    printvar( map(factor,bvals), vtype=VECTOR, vsym="b",
      fid=pfid, wid=pwid ):
    printvar( "\nSubstituting this expression for q[3] into Equation (2)"
      " and solving for q[1] yields:\n", fid=pfid, wid=pwid ):
    printvar( q1soln, vtype=SCALAR, vsym="q[1]",
      fid=pfid, wid=pwid ):
    printvar( "\nwhere\n\n", fid=pfid, wid=pwid ):
    printvar( avals, vtype=VECTOR, vsym="a", fid=pfid, wid=pwid ):
  end if:

  # Update vq, ybar, vbar, and psi_py
  vq := convert(convert(indets(vq) minus {q[1],q[3]},list),Vector[column]):
  vbar := subs(q[1]=q1soln,q[3]=q3soln,vbar):
  ybar := subs(q[1]=q1soln,q[3]=q3soln,ybar):
  psi := subs(q[1]=q1soln,q[3]=q3soln,psi):

  if pverb then
    printvar( "\nWe can now update the forward map psi_py"
      " and the steady state reaction velocity vector, vbar\n.",
      fid=pfid, wid=pwid ):
    printvar( psi, vtype=MAP, fid=pfid, wid=pwid ):
    printvar( "\nand\n", fid=pfid, wid=pwid ):
    printvar( vbar, vtype=VECTOR, psym="v", fid=pfid, wid=pwid ):
  end if:

  # Unsub, if requested
  if punsub then
    vbar,psi := Unsub( vv, vp, vq, vy, kx, py, ybar, 
      PI, verbose=pverb, wid=pwid, fid=pfid ):
  end if:

  # Verify steady state
  if pverb then
    printvar(sprintf("\nVerify steady state. xdot ="
      " S * vbar, where\n"),fid=pfid,wid=pwid ): 
    printvar( simplify(LinearAlgebra[Multiply](S,vbar)), 
      vtype=VECTOR, vsym="xdot", fid=pfid, wid=pwid ):
  end if:
  
  return(vbar,psi):

end proc: